import dataclasses
from typing import List

from dataclasses_json import DataClassJsonMixin

from helixtelemetry.telemetry.context.telemetry_context import TelemetryContext
from helixtelemetry.telemetry.structures.telemetry_parent import TelemetryParent


@dataclasses.dataclass
class ConsoleTelemetryHistoryItem(DataClassJsonMixin):
    context: TelemetryContext

    telemetry_parent: TelemetryParent

    name: str

    children: List["ConsoleTelemetryHistoryItem"] = dataclasses.field(
        default_factory=list
    )

    @classmethod
    def from_telemetry_context(
        cls,
        *,
        name: str,
        telemetry_context: TelemetryContext,
        telemetry_parent: TelemetryParent,
    ) -> "ConsoleTelemetryHistoryItem":
        return cls(
            context=telemetry_context,
            children=[],
            name=name,
            telemetry_parent=telemetry_parent,
        )



import dataclasses
from typing import Optional, List, Union, Mapping

from dataclasses_json import DataClassJsonMixin

from helixtelemetry.telemetry.structures.telemetry_attribute_value import (
    TelemetryAttributeValue,
)
from helixtelemetry.telemetry.structures.telemetry_tracers import TelemetryTracer


@dataclasses.dataclass
class TelemetryContext(DataClassJsonMixin):
    provider: str
    """ Provider for the telemetry context """

    service_name: str
    """ Service name for the telemetry context """

    service_namespace: str
    """ Service namespace for the telemetry context.  Included in traces and metrics """

    instance_name: str
    """ Instance name for the telemetry context.  Included in traces and metrics """

    environment: str
    """ Environment for the telemetry context """

    attributes: Optional[Mapping[str, TelemetryAttributeValue]]
    """ Additional attributes to include in telemetry """

    log_level: Optional[Union[int, str]]
    """ Log level for the telemetry context """

    trace_all_calls: Optional[List[TelemetryTracer]] = None
    """ Whether to Trace certain calls like aiohttp, pymysql, etc """

    tracer_endpoint: Optional[str] = None
    """ Tracer endpoint for the telemetry context """

    metrics_endpoint: Optional[str] = None
    """ Metrics endpoint for the telemetry context """

    @staticmethod
    def get_null_context() -> "TelemetryContext":
        """
        Get a null telemetry context

        :return: a null telemetry context
        """
        return TelemetryContext(
            provider="NullTelemetry",
            service_name="",
            environment="",
            attributes=None,
            log_level=None,
            instance_name="",
            service_namespace="",
        )

    def copy(self) -> "TelemetryContext":
        """
        Create a copy of the telemetry context

        :return: a copy of the telemetry context
        """
        return TelemetryContext(
            provider=self.provider,
            service_name=self.service_name,
            environment=self.environment,
            trace_all_calls=self.trace_all_calls,
            attributes=self.attributes,
            log_level=self.log_level,
            instance_name=self.instance_name,
            service_namespace=self.service_namespace,
        )

    def create_child_context(self) -> "TelemetryContext":
        """
        Create a child telemetry context

        :return: a child telemetry context
        """
        telemetry_context = self.copy()
        return telemetry_context



from typing import Mapping, List, Dict

from helixtelemetry.telemetry.structures.telemetry_attribute_value import (
    TelemetryAttributeValue,
    TelemetryAttributeValueWithoutNone,
)


def append_mappings(
    mappings: List[Mapping[str, TelemetryAttributeValue] | None],
) -> Mapping[str, TelemetryAttributeValueWithoutNone]:
    """
    Append mappings

    :param mappings: List of mappings
    :return: Aggregated mappings
    """
    assert mappings

    final_attributes: Dict[str, TelemetryAttributeValue] = {}
    for mapping in mappings:
        if mapping:
            final_attributes.update(mapping)

    final_attributes_without_none = remove_null_values(final_attributes)

    return final_attributes_without_none


def remove_null_values(
    final_attributes: Mapping[str, TelemetryAttributeValue],
) -> Mapping[str, TelemetryAttributeValueWithoutNone]:
    # tracer cannot handle None attributes so ignore attributes that are None
    final_attributes_without_none: Mapping[str, TelemetryAttributeValueWithoutNone] = {
        k: v
        for k, v in (final_attributes or {}).items()
        if v is not None and type(v) in [bool, str, bytes, int, float]
    }
    return final_attributes_without_none



from abc import ABC, abstractmethod
from typing import (
    Optional,
    Dict,
    Any,
    Mapping,
)

from helixtelemetry.telemetry.structures.telemetry_attribute_value import (
    TelemetryAttributeValue,
)
from helixtelemetry.telemetry.structures.telemetry_parent import TelemetryParent
from helixtelemetry.telemetry.utilities.mapping_appender import append_mappings


class TelemetrySpanWrapper(ABC):
    def __init__(
        self,
        *,
        name: str,
        attributes: Optional[Mapping[str, TelemetryAttributeValue]],
        telemetry_parent: Optional[TelemetryParent],
    ) -> None:
        """
        Wrapper for telemetry span

        :param name:
        :param telemetry_parent:
        """
        self.name: str = name
        self.attributes: Optional[Mapping[str, TelemetryAttributeValue]] = attributes
        self._telemetry_parent: Optional[TelemetryParent] = telemetry_parent

    @property
    @abstractmethod
    def trace_id(self) -> Optional[str]: ...

    @property
    @abstractmethod
    def span_id(self) -> Optional[str]: ...

    def create_child_telemetry_parent(
        self,
        *,
        attributes: Mapping[str, TelemetryAttributeValue] | None = None,
        include_parent_attributes: bool = True,
    ) -> TelemetryParent | None:
        """
        Creates a copy of the telemetry parent with the new trace and span ids


        :return:
        """
        if not self._telemetry_parent:
            return None

        return self._telemetry_parent.create_child_telemetry_parent(
            attributes=(
                append_mappings([self.attributes, attributes])
                if include_parent_attributes
                else attributes
            ),
        )

    @abstractmethod
    def set_attributes(self, attributes: Dict[str, Any]) -> None: ...

    """
    This can be used AFTER a span is created to add attributes to it
    
    :param attributes: new attributes to add to the span
    """

    @abstractmethod
    def end(self, *, end_time: int) -> None: ...

    """
    This can be used to manually end a span with a specific end time
    
    :param end_time: end time of the span
    """



import logging
import uuid
from contextlib import asynccontextmanager, contextmanager
from logging import Logger
from typing import Optional, Dict, Any, AsyncGenerator, Generator, Mapping, List

from helixtelemetry.telemetry.context.telemetry_context import TelemetryContext
from helixtelemetry.telemetry.metrics.telemetry_counter import TelemetryCounter
from helixtelemetry.telemetry.metrics.telemetry_histogram_counter import (
    TelemetryHistogram,
)
from helixtelemetry.telemetry.metrics.telemetry_up_down_counter import (
    TelemetryUpDownCounter,
)
from helixtelemetry.telemetry.providers.telemetry import Telemetry
from helixtelemetry.telemetry.spans.console_telemetry_span_wrapper import (
    ConsoleTelemetrySpanWrapper,
)
from helixtelemetry.telemetry.spans.telemetry_span_wrapper import TelemetrySpanWrapper
from helixtelemetry.telemetry.structures.telemetry_attribute_value import (
    TelemetryAttributeValue,
)
from helixtelemetry.telemetry.structures.telemetry_parent import TelemetryParent


class TelemetrySpanCreator:
    def __init__(
        self,
        *,
        telemetry: Optional[Telemetry],
        log_level: str = "DEBUG",
    ) -> None:
        """
        Create a telemetry span creator that can create a telemetry span if telemetry is available else return a null context

        :param telemetry: Optional telemetry object
        """

        # Unique instance identifier
        self._instance_id = str(uuid.uuid4())

        assert telemetry is not None
        self.telemetry = telemetry

        self._logger: Logger = logging.getLogger(
            __name__,
        )
        # get_logger sets the log level to the environment variable LOGLEVEL if it exists
        self._logger.setLevel(log_level)

    def __getstate__(self) -> Dict[str, Any]:
        raise NotImplementedError(
            "Serialization of TelemetrySpanCreator is not supported.  Did you accidentally try to send this object to a Spark worker?"
        )

    @asynccontextmanager
    async def create_telemetry_span_async(
        self,
        *,
        name: str,
        attributes: Optional[Mapping[str, TelemetryAttributeValue]],
        telemetry_parent: Optional[TelemetryParent],
        start_time: int | None = None,
        add_metadata: Optional[List[str]] = None,
        add_telemetry_attribute: Optional[List[str]] = None,
    ) -> AsyncGenerator[TelemetrySpanWrapper, None]:
        """
        Create a telemetry span if telemetry is available else return a null context

        :param name: name of the span
        :param attributes:  optional attributes to add to the span
        :param telemetry_parent: telemetry parent
        :param start_time: start time
        :return: AsyncGenerator[TelemetrySpanWrapper, None]
        """

        if self.telemetry is not None:
            span: TelemetrySpanWrapper
            async with self.telemetry.trace_async(
                name=name,
                attributes=attributes,
                telemetry_parent=telemetry_parent,
                start_time=start_time,
                add_metadata=add_metadata,
                add_telemetry_attribute=add_telemetry_attribute,
            ) as span:
                yield span
        else:
            yield ConsoleTelemetrySpanWrapper(
                name=name,
                attributes=attributes,
                telemetry_context=TelemetryContext.get_null_context(),
                telemetry_parent=telemetry_parent,
            )

    @contextmanager
    def create_telemetry_span(
        self,
        *,
        name: str,
        attributes: Optional[Mapping[str, TelemetryAttributeValue]],
        telemetry_parent: Optional[TelemetryParent],
        start_time: int | None = None,
    ) -> Generator[TelemetrySpanWrapper, None, None]:
        """
        Create a telemetry span if telemetry is available else return a null context

        :param name: name of the span
        :param attributes:  optional attributes to add to the span
        :param telemetry_parent: telemetry parent
        :param start_time: start time
        :return: Generator[TelemetrySpanWrapper, None, None]
        """
        if self.telemetry is not None:
            span: TelemetrySpanWrapper
            with self.telemetry.trace(
                name=name,
                attributes=attributes,
                telemetry_parent=telemetry_parent,
                start_time=start_time,
            ) as span:
                yield span
        else:
            yield ConsoleTelemetrySpanWrapper(
                name=name,
                attributes=attributes,
                telemetry_context=TelemetryContext.get_null_context(),
                telemetry_parent=telemetry_parent,
            )

    async def flush_async(self) -> None:
        """
        Flush the telemetry

        :return: None
        """
        if self.telemetry:
            await self.telemetry.flush_async()

    def get_telemetry_counter(
        self,
        *,
        name: str,
        unit: str,
        description: str,
        telemetry_parent: Optional[TelemetryParent],
        attributes: Optional[Mapping[str, TelemetryAttributeValue]] = None,
        add_metadata: Optional[List[str]] = None,
    ) -> TelemetryCounter:
        """
        Get a counter metric

        :param name: Name of the counter
        :param unit: Unit of the counter
        :param description: Description
        :param attributes: Optional attributes
        :param telemetry_parent: telemetry parent
        :return: The Counter metric
        """
        return self.telemetry.get_counter(
            name=name,
            unit=unit,
            description=description,
            attributes=attributes,
            telemetry_parent=telemetry_parent,
            add_metadata=add_metadata,
        )

    def get_telemetry_up_down_counter(
        self,
        *,
        name: str,
        unit: str,
        description: str,
        telemetry_parent: Optional[TelemetryParent],
        attributes: Optional[Dict[str, Any]] = None,
    ) -> TelemetryUpDownCounter:
        """
        Get an up_down_counter metric

        :param name: Name of the up_down_counter
        :param unit: Unit of the up_down_counter
        :param description: Description
        :param attributes: Optional attributes
        :param telemetry_parent: telemetry parent
        :return: The Counter metric
        """
        return self.telemetry.get_up_down_counter(
            name=name,
            unit=unit,
            description=description,
            attributes=attributes,
            telemetry_parent=telemetry_parent,
        )

    def get_telemetry_histogram(
        self,
        *,
        name: str,
        unit: str,
        description: str,
        telemetry_parent: Optional[TelemetryParent],
        attributes: Optional[Mapping[str, TelemetryAttributeValue]] = None,
    ) -> TelemetryHistogram:
        """
        Get a histograms metric

        :param name: Name of the histograms
        :param unit: Unit of the histograms
        :param description: Description
        :param attributes: Optional attributes
        :param telemetry_parent: telemetry parent
        :return: The Counter metric
        """
        return self.telemetry.get_histogram(
            name=name,
            unit=unit,
            description=description,
            attributes=attributes,
            telemetry_parent=telemetry_parent,
        )



from typing import Optional, Union, Mapping

from opentelemetry.context import Context
from opentelemetry.metrics import Histogram

from helixtelemetry.telemetry.structures.telemetry_attribute_value import (
    TelemetryAttributeValue,
    TelemetryAttributeValueWithoutNone,
)
from helixtelemetry.telemetry.structures.telemetry_parent import TelemetryParent
from helixtelemetry.telemetry.utilities.mapping_appender import append_mappings


class TelemetryHistogram:
    """
    This class wraps the OpenTelemetry Histogram class and adds the supplied attributes every time a metric is recorded

    """

    def __init__(
        self,
        *,
        histogram: Histogram,
        attributes: Optional[Mapping[str, TelemetryAttributeValue]],
        telemetry_parent: Optional[TelemetryParent],
    ) -> None:
        assert histogram
        self._histogram: Histogram = histogram
        self._attributes: Optional[Mapping[str, TelemetryAttributeValue]] = attributes
        self._telemetry_parent: Optional[TelemetryParent] = telemetry_parent

    def record(
        self,
        amount: Union[int, float],
        attributes: Optional[Mapping[str, TelemetryAttributeValue]] = None,
        context: Optional[Context] = None,
    ) -> None:
        combined_attributes: Mapping[str, TelemetryAttributeValueWithoutNone] = (
            append_mappings(
                [
                    self._attributes,
                    self._telemetry_parent.attributes if self._telemetry_parent else {},
                    attributes,
                ]
            )
        )

        self._histogram.record(
            amount=amount, attributes=combined_attributes, context=context
        )



import asyncio
import os
import time
from typing import Optional, Any, Dict

from opentelemetry import trace
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import (
    BatchSpanProcessor,
    ConsoleSpanExporter,  # For debugging
)
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter
from opentelemetry.sdk.resources import Resource
from opentelemetry.semconv.resource import ResourceAttributes
from opentelemetry.trace import Span, SpanContext, TraceFlags, NonRecordingSpan
from opentelemetry.trace.propagation.tracecontext import TraceContextTextMapPropagator

service_name: str = "default-service"
endpoint: str = "otel-collector:4317"
environment: str = "development"


def configure_tracer(
    service_name: str,
    endpoint: str,
    environment: str,
) -> Optional[TracerProvider]:
    try:
        # Create a resource with service details
        resource = Resource.create(
            {
                ResourceAttributes.SERVICE_NAME: service_name,
                ResourceAttributes.DEPLOYMENT_ENVIRONMENT: environment,
            }
        )

        # Create OTLP exporter with full configuration
        otlp_exporter = OTLPSpanExporter(
            endpoint=f"http://{endpoint}",
            # Add timeout and other parameters if needed
            timeout=10,  # 10 seconds timeout
        )

        # Create tracer provider with resource
        provider = TracerProvider(resource=resource)

        # Add both OTLP and Console exporters for debugging
        otlp_processor = BatchSpanProcessor(otlp_exporter)
        console_processor = BatchSpanProcessor(ConsoleSpanExporter())

        provider.add_span_processor(otlp_processor)
        provider.add_span_processor(console_processor)

        # Set global tracer provider
        trace.set_tracer_provider(provider)

        return provider

    except Exception as e:
        print(f"Error configuring tracer: {e}")
        return None


async def run_sub_operation(
    trace_id: int, span_id: int, carrier: Dict[str, Any]
) -> None:
    """
    Run the sub-operation in a separate thread.

    Args:

    """
    print(f"Running sub-operation: {trace_id}, {span_id}, {carrier}")

    provider: TracerProvider | None = configure_tracer(
        service_name=service_name, endpoint=endpoint, environment=environment
    )

    assert provider is not None, "Tracer provider should be initialized"

    # Creates a tracer from the global tracer provider
    tracer = trace.get_tracer(service_name)

    trace_id_hex = f"{trace_id:032x}"
    span_id_hex = f"{span_id:016x}"

    trace_id_int = int(trace_id_hex, 16)
    span_id_int = int(span_id_hex, 16)

    span_context = SpanContext(
        trace_id=trace_id_int,
        span_id=span_id_int,
        is_remote=True,
        trace_flags=TraceFlags(0x01),
    )
    ctx = trace.set_span_in_context(NonRecordingSpan(span_context))

    with tracer.start_as_current_span("sub-operation", context=ctx) as sub_span:
        sub_span.add_event("sub-operation-event")
        time.sleep(0.05)  # Simulate sub-operation


async def test_open_telemetry_raw_multi_thread() -> None:
    # Set environment variables explicitly
    os.environ["OTEL_SERVICE_NAME"] = service_name
    os.environ["OTEL_EXPORTER_OTLP_ENDPOINT"] = f"http://{endpoint}"
    os.environ["OTEL_TRACES_EXPORTER"] = "otlp"
    os.environ["OTEL_PYTHON_LOGGING_AUTO_INSTRUMENTATION_ENABLED"] = "true"

    # Configure tracer
    provider: TracerProvider | None = configure_tracer(
        service_name=service_name, endpoint=endpoint, environment=environment
    )

    assert provider is not None, "Tracer provider should be initialized"

    # Creates a tracer from the global tracer provider
    tracer = trace.get_tracer(service_name)

    # Create a more complex trace with multiple events and attributes
    span: Span
    with tracer.start_as_current_span("main-operation") as span:
        span.set_attribute("operation.type", "test")
        span.add_event("operation-started")

        carrier: Dict[str, Any] = {}
        # Write the current context into the carrier.
        TraceContextTextMapPropagator().inject(carrier)

        span_context: SpanContext = span.get_span_context()
        trace_id: int = span_context.trace_id
        span_id: int = span_context.span_id

        print(f"Running main-operation: {trace_id}, {span_id}, {carrier}")

        try:
            # Simulate some work
            time.sleep(0.1)  # Simulate processing time
            print("Performing some work...")

            # Create a thread for the sub-operation
            # sub_thread = threading.Thread(
            #     target=run_sub_operation,
            #     args=(trace_id, span_id, carrier),
            #     daemon=True  # Ensures thread will exit when main thread exits
            # )
            # sub_thread.start()
            #
            # # Wait for the sub-thread to complete
            # sub_thread.join()

            # Create a task for the sub-operation
            sub_task = asyncio.create_task(
                run_sub_operation(trace_id, span_id, carrier)
            )

            # Wait for the sub-task to complete
            await sub_task

            span.add_event("operation-completed")
        except Exception as e:
            span.record_exception(e)
            span.set_status(trace.Status(trace.StatusCode.ERROR))

    # Force flush to ensure traces are sent
    provider.force_flush()
    time.sleep(1)  # Give time for export

    # Shutdown properly
    provider.shutdown()



from __future__ import annotations  # For self-referencing type hints
import dataclasses
from typing import Optional, Mapping

from dataclasses_json import DataClassJsonMixin

from helixtelemetry.telemetry.context.telemetry_context import TelemetryContext
from helixtelemetry.telemetry.structures.telemetry_attribute_value import (
    TelemetryAttributeValue,
)
from helixtelemetry.telemetry.utilities.mapping_appender import append_mappings


@dataclasses.dataclass
class TelemetryParent(DataClassJsonMixin):
    name: str
    """ Name of the telemetry parent """

    trace_id: Optional[str]
    """ Trace ID for the telemetry context """

    span_id: Optional[str]
    """ Span ID for the telemetry context """

    attributes: Optional[Mapping[str, TelemetryAttributeValue]]
    """ Attributes for the telemetry parent to inherit by children """

    telemetry_context: TelemetryContext
    """ Telemetry context for the telemetry parent """

    def create_child_telemetry_parent(
        self,
        *,
        attributes: Mapping[str, TelemetryAttributeValue] | None = None,
        include_parent_attributes: bool = True,
    ) -> TelemetryParent:
        """
        Creates a copy of the telemetry parent with the new trace and span ids


        :return:
        """
        child_telemetry_context = self.telemetry_context.create_child_context()

        child_telemetry_parent: TelemetryParent = TelemetryParent(
            trace_id=self.trace_id,
            span_id=self.span_id,
            name=self.name,
            attributes=(
                append_mappings([self.attributes, attributes])
                if include_parent_attributes
                else attributes
            ),
            telemetry_context=child_telemetry_context,
        )

        return child_telemetry_parent

    @classmethod
    def get_null_parent(cls) -> TelemetryParent:
        """
        Get a null telemetry context

        :return: a null telemetry context
        """
        return TelemetryParent(
            name="Null",
            trace_id=None,
            span_id=None,
            attributes=None,
            telemetry_context=TelemetryContext.get_null_context(),
        )



import asyncio
import os
import time
from typing import Optional, Any, Dict

from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import (
    BatchSpanProcessor,
    ConsoleSpanExporter,  # For debugging
)
from opentelemetry.semconv.resource import ResourceAttributes
from opentelemetry.trace.propagation.tracecontext import TraceContextTextMapPropagator

from helixtelemetry.telemetry.context.telemetry_context import TelemetryContext
from helixtelemetry.telemetry.providers.open_telemetry import OpenTelemetry
from helixtelemetry.telemetry.spans.telemetry_span_wrapper import TelemetrySpanWrapper

service_name: str = "default-service"
endpoint: str = "otel-collector:4317"
environment: str = "development"


def configure_tracer() -> Optional[TracerProvider]:
    try:
        # Create a resource with service details
        resource = Resource.create(
            {
                ResourceAttributes.SERVICE_NAME: service_name,
                ResourceAttributes.DEPLOYMENT_ENVIRONMENT: environment,
            }
        )

        # Create OTLP exporter with full configuration
        otlp_exporter = OTLPSpanExporter(
            endpoint=f"http://{endpoint}",
            # Add timeout and other parameters if needed
            timeout=10,  # 10 seconds timeout
        )

        # Create tracer provider with resource
        provider = TracerProvider(resource=resource)

        # Add both OTLP and Console exporters for debugging
        otlp_processor = BatchSpanProcessor(otlp_exporter)
        console_processor = BatchSpanProcessor(ConsoleSpanExporter())

        provider.add_span_processor(otlp_processor)
        provider.add_span_processor(console_processor)

        # Set global tracer provider
        trace.set_tracer_provider(provider)

        return provider

    except Exception as e:
        print(f"Error configuring tracer: {e}")
        return None


async def run_sub_operation(
    trace_id: Optional[str], span_id: Optional[str], carrier: Dict[str, Any]
) -> None:
    """
    Run the sub-operation in a separate thread.

    Args:

    """
    print(f"Running sub-operation: {trace_id}, {span_id}, {carrier}")

    telemetry_context = TelemetryContext(
        service_name=service_name,
        environment=environment,
        provider=OpenTelemetry.telemetry_provider,
        attributes=None,
        log_level=None,
        instance_name="sub-operation",
        service_namespace="MyNamespace",
    )
    telemetry = OpenTelemetry(
        telemetry_context=telemetry_context,
        write_telemetry_to_console=True,
        log_level="DEBUG",
    )

    # Create a more complex trace with multiple events and attributes
    span: TelemetrySpanWrapper
    async with telemetry.trace_async(
        name="main-operation", telemetry_parent=None
    ) as sub_span:
        time.sleep(0.05)  # Simulate sub-operation


async def test_open_telemetry_multi_thread() -> None:
    # Set environment variables explicitly
    os.environ["OTEL_SERVICE_NAME"] = service_name
    os.environ["OTEL_EXPORTER_OTLP_ENDPOINT"] = f"http://{endpoint}"
    os.environ["OTEL_TRACES_EXPORTER"] = "otlp"
    os.environ["OTEL_PYTHON_LOGGING_AUTO_INSTRUMENTATION_ENABLED"] = "true"

    telemetry_context = TelemetryContext(
        service_name=service_name,
        environment=environment,
        provider=OpenTelemetry.telemetry_provider,
        attributes=None,
        log_level=None,
        instance_name="main-operation",
        service_namespace="MyNamespace",
    )
    telemetry = OpenTelemetry(
        telemetry_context=telemetry_context,
        write_telemetry_to_console=True,
        log_level="DEBUG",
    )

    # Create a more complex trace with multiple events and attributes
    span: TelemetrySpanWrapper
    async with telemetry.trace_async(
        name="main-operation", telemetry_parent=None
    ) as span:

        carrier: Dict[str, Any] = {}
        # Write the current context into the carrier.
        TraceContextTextMapPropagator().inject(carrier)

        trace_id: Optional[str] = span.trace_id
        span_id: Optional[str] = span.span_id

        print(f"Running main-operation: {trace_id}, {span_id}, {carrier}")

        try:
            # Simulate some work
            time.sleep(0.1)  # Simulate processing time
            print("Performing some work...")

            # Create a task for the sub-operation
            sub_task = asyncio.create_task(
                run_sub_operation(trace_id, span_id, carrier)
            )

            # Wait for the sub-task to complete
            await sub_task

        except Exception as e:
            print(f"Error running main-operation: {e}")

    # Force flush to ensure traces are sent
    await telemetry.flush_async()
    time.sleep(1)  # Give time for export

    # Shutdown properly
    await telemetry.shutdown_async()



from typing import Optional, Dict, Union, Mapping

from opentelemetry.context import Context
from opentelemetry.metrics import Counter

from helixtelemetry.telemetry.structures.telemetry_attribute_value import (
    TelemetryAttributeValue,
    TelemetryAttributeValueWithoutNone,
)
from helixtelemetry.telemetry.structures.telemetry_parent import TelemetryParent
from helixtelemetry.telemetry.utilities.mapping_appender import append_mappings


class TelemetryCounter:
    """
    This class wraps the OpenTelemetry Counter class and adds the supplied attributes every time a metric is recorded

    """

    def __init__(
        self,
        *,
        counter: Counter,
        attributes: Optional[Mapping[str, TelemetryAttributeValue]],
        telemetry_parent: Optional[TelemetryParent],
    ) -> None:
        assert counter
        self._counter: Counter = counter
        self._attributes: Optional[Mapping[str, TelemetryAttributeValue]] = attributes
        self._telemetry_parent: Optional[TelemetryParent] = telemetry_parent

    def add(
        self,
        amount: Union[int, float],
        attributes: Optional[Dict[str, bool | str | bytes | int | float | None]] = None,
        context: Optional[Context] = None,
    ) -> None:
        combined_attributes: Mapping[str, TelemetryAttributeValueWithoutNone] = (
            append_mappings(
                [
                    self._attributes,
                    self._telemetry_parent.attributes if self._telemetry_parent else {},
                    attributes,
                ]
            )
        )

        self._counter.add(
            amount=amount, attributes=combined_attributes, context=context
        )



import os
import time
from typing import Optional

from opentelemetry import trace
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import (
    BatchSpanProcessor,
    ConsoleSpanExporter,  # For debugging
)
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter
from opentelemetry.sdk.resources import Resource
from opentelemetry.semconv.resource import ResourceAttributes
from opentelemetry.trace import Span


def configure_tracer(
    service_name: str = "default-service",
    endpoint: str = "localhost:4317",
    environment: str = "development",
) -> Optional[TracerProvider]:
    try:
        # Create a resource with service details
        resource = Resource.create(
            {
                ResourceAttributes.SERVICE_NAME: service_name,
                ResourceAttributes.DEPLOYMENT_ENVIRONMENT: environment,
            }
        )

        # Create OTLP exporter with full configuration
        otlp_exporter = OTLPSpanExporter(
            endpoint=f"http://{endpoint}",
            # Add timeout and other parameters if needed
            timeout=10,  # 10 seconds timeout
        )

        # Create tracer provider with resource
        provider = TracerProvider(resource=resource)

        # Add both OTLP and Console exporters for debugging
        otlp_processor = BatchSpanProcessor(otlp_exporter)
        console_processor = BatchSpanProcessor(ConsoleSpanExporter())

        provider.add_span_processor(otlp_processor)
        provider.add_span_processor(console_processor)

        # Set global tracer provider
        trace.set_tracer_provider(provider)

        return provider

    except Exception as e:
        print(f"Error configuring tracer: {e}")
        return None


def test_open_telemetry_raw() -> None:
    # Detailed configuration
    service_name: str = "demo-service"
    # endpoint: str = "localhost:4317"
    endpoint: str = "otel-collector:4317"
    environment: str = "development"

    # Set environment variables explicitly
    os.environ["OTEL_SERVICE_NAME"] = service_name
    os.environ["OTEL_EXPORTER_OTLP_ENDPOINT"] = f"http://{endpoint}"
    os.environ["OTEL_TRACES_EXPORTER"] = "otlp"
    os.environ["OTEL_PYTHON_LOGGING_AUTO_INSTRUMENTATION_ENABLED"] = "true"

    # Configure tracer
    provider: TracerProvider | None = configure_tracer(
        service_name=service_name, endpoint=endpoint, environment=environment
    )

    assert provider is not None, "Tracer provider should be initialized"

    # Creates a tracer from the global tracer provider
    tracer = trace.get_tracer("my.tracer.name")

    # Create a more complex trace with multiple events and attributes
    span: Span
    with tracer.start_as_current_span("main-operation") as span:
        span.set_attribute("operation.type", "test")
        span.add_event("operation-started")

        try:
            # Simulate some work
            time.sleep(0.1)  # Simulate processing time
            print("Performing some work...")

            with tracer.start_as_current_span("sub-operation") as sub_span:
                sub_span.add_event("sub-operation-event")
                time.sleep(0.05)  # Simulate sub-operation

            span.add_event("operation-completed")
        except Exception as e:
            span.record_exception(e)
            span.set_status(trace.Status(trace.StatusCode.ERROR))

    # Force flush to ensure traces are sent
    provider.force_flush()
    time.sleep(1)  # Give time for export

    # Shutdown properly
    provider.shutdown()
